@@ -76,7 +76,7 @@ couchdb_document_get (CouchDB *couchdb,
 		      const char *docid,
 		      GError **error)
 {
-	char *url;
+	char *url, *encoded_docid;
 	JsonParser *parser;
 	CouchDBDocument *document = NULL;
 
@@ -84,7 +84,8 @@ couchdb_document_get (CouchDB *couchdb,
 	g_return_val_if_fail (dbname != NULL, NULL);
 	g_return_val_if_fail (docid != NULL, NULL);
 
-	url = g_strdup_printf ("%s/%s/%s", couchdb->hostname, dbname, docid);
+	encoded_docid = soup_uri_encode (docid, NULL);
+	url = g_strdup_printf ("%s/%s/%s", couchdb->hostname, dbname, encoded_docid);
 	parser = send_message_and_parse (couchdb, SOUP_METHOD_GET, url, NULL, error);
 	if (parser) {
 		document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
@@ -95,6 +96,7 @@ couchdb_document_get (CouchDB *couchdb,
 		g_object_unref (G_OBJECT (parser));
 	}
 
+	g_free (encoded_docid);
 	g_free (url);
 
 	return document;
@@ -116,8 +118,13 @@ couchdb_document_put (CouchDBDocument *document,
 	id = couchdb_document_get_id (document);
 	body = couchdb_document_to_string (document);
 	if (id) {
-		url = g_strdup_printf ("%s/%s/%s", document->couchdb->hostname, dbname, id);
+		char *encoded_docid;
+
+		encoded_docid = soup_uri_encode (id, NULL);
+		url = g_strdup_printf ("%s/%s/%s", document->couchdb->hostname, dbname, encoded_docid);
 		parser = send_message_and_parse (document->couchdb, SOUP_METHOD_PUT, url, body, error);
+
+		g_free (encoded_docid);
 	} else {
 		url = g_strdup_printf ("%s/%s/", document->couchdb->hostname, dbname);
 		parser = send_message_and_parse (document->couchdb, SOUP_METHOD_POST, url, body, error);
@@ -137,6 +144,11 @@ couchdb_document_put (CouchDBDocument *document,
 			document->dbname = g_strdup (dbname);
 		}
 
+		if (id)
+			g_signal_emit_by_name (document->couchdb, "document_updated", dbname, document);
+		else
+			g_signal_emit_by_name (document->couchdb, "document_created", dbname, document);
+
 		result = TRUE;
 	}
 
@@ -167,7 +179,7 @@ couchdb_document_delete (CouchDBDocument *document, GError **error)
 	if (parser) {
 		g_object_unref (G_OBJECT (parser));
 
-		result = TRUE;
+		g_signal_emit_by_name (document->couchdb, "document_deleted", document->dbname, id);
 	}
 
 	g_free (url);
@@ -182,9 +194,11 @@ couchdb_document_get_id (CouchDBDocument *document)
 
 	if (document->root_node &&
 	    json_node_get_node_type (document->root_node) == JSON_NODE_OBJECT) {
-		return json_object_get_string_member (
-			json_node_get_object (document->root_node),
-			"_id");
+		if (json_object_has_member (json_node_get_object (document->root_node),
+					    "_id"))
+			return json_object_get_string_member (
+				json_node_get_object (document->root_node),
+				"_id");
 	}
 
 	return NULL;
@@ -279,6 +293,9 @@ couchdb_document_get_boolean_field (CouchDBDocument *document, const char *field
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
 	g_return_val_if_fail (field != NULL, FALSE);
 
+	if (!json_object_has_member (json_node_get_object (document->root_node), field))
+		return FALSE;
+
 	return json_object_get_boolean_member (json_node_get_object (document->root_node),
 					       field);
 }
@@ -300,6 +317,9 @@ couchdb_document_get_int_field (CouchDBDocument *document, const char *field)
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), -1);
 	g_return_val_if_fail (field != NULL, -1);
 
+	if (!json_object_has_member (json_node_get_object (document->root_node), field))
+		return 0;
+
 	return json_object_get_int_member (json_node_get_object (document->root_node),
 					   field);
 }
@@ -321,6 +341,8 @@ couchdb_document_get_double_field (CouchDBDocument *document, const char *field)
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), -1);
 	g_return_val_if_fail (field != NULL, -1);
 
+	if (!json_object_has_member (json_node_get_object (document->root_node), field))
+		return 0.0;
 	return json_object_get_double_member (json_node_get_object (document->root_node),
 					      field);
 }
@@ -342,6 +364,9 @@ couchdb_document_get_string_field (CouchDBDocument *document, const char *field)
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (field != NULL, NULL);
 
+	if (!json_object_has_member (json_node_get_object (document->root_node), field))
+		return NULL;
+
 	return json_object_get_string_member (json_node_get_object (document->root_node),
 					      field);
 }
@@ -368,6 +393,9 @@ couchdb_document_get_struct_field (CouchDBDocument *document, const char *field)
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
 	g_return_val_if_fail (field != NULL, NULL);
 
+	if (!json_object_has_member (json_node_get_object (document->root_node), field))
+		return NULL;
+
 	return couchdb_struct_field_new_from_json_object (
 		json_object_ref (json_object_get_object_member (json_node_get_object (document->root_node),
 								field)));
